#
#  FILE NAME:
#      import_helpers.py
#
#  REFERENCE(S):
#      * http://docs.python.org/library/functions.html#__import__
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library modules:
import inspect
import logging
import re
import sys

# P3 modules:
from p3.base.exception import BaseModuleError
from p3.base.string_helpers import safe_fmt


_LOG = logging.getLogger( "p3.base" )


def import_module( module_path ):
    if not isinstance( module_path, basestring ):
        raise TypeError( "module_path argument is expected to be a string." )

    if not module_path:
        raise ValueError( "module_path is empty." )

    try:
        __import__( module_path, globals(), locals(), [], -1 )
    except ImportError as e:
        _LOG.error( "Following exception occured when attempted to import '%s' module:\n%s",
                    module_path, str(e) )
        raise

    return sys.modules[module_path]


def import_object( object_path ):
    if not isinstance( object_path, basestring ):
        raise TypeError( "object_path argument is expected to be a string." )

    match = re.match( "^(?P<module>(?:\w+\.)*\w+)\.(?P<object>\w+)$", object_path )

    if match is None:
        msg = safe_fmt( "Couldn't parse object path ('%s')", object_path )
        _LOG.error( msg )
        raise BaseModuleError( BaseModuleError.codes.invalid_import_path, msg )

    try:
        module = __import__(
            match.group('module'), globals(), locals(), [match.group('object')], -1 )
    except ImportError as e:
        _LOG.info( "Following exception occured when attempted to import '%s' object:\n%s",
                    object_path, str(e) )
        raise

    if match.group('object') not in module.__dict__:
        msg = safe_fmt( "Couldn't import '%s' object from the '%s' module.",
                        match.group('object'), match.group('module') )
        _LOG.error( msg )
        raise ImportError( msg )

    return module.__dict__[match.group('object')]


def is_module_available( module_path ):
    try:
        import_module( module_path )
        return True
    except ImportError as e:
        return False


def challenge_module( module_path, extra_msg=None ):
    try:
        import_module( module_path )
    except ImportError as e:
        if extra_msg is None:
            caller = inspect.currentframe().f_back
            extra_msg = 'expected by the {0:s} module'.format( caller.f_globals['__name__'] )

        msg = "Couldn't import required '{0:s}' module ({1:s}).".format( module_path, extra_msg )

        raise BaseModuleError( BaseModuleError.codes.required_module_not_available, msg )
