#
#  FILE NAME:
#      settings.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# P3 imports:
from p3.base.string_helpers import \
    is_all_caps_identifier_string, is_internal_identifier_string, safe_fmt
from p3.conf.error import ConfModuleError



class Settings( object ):
    @classmethod
    def create_locked( cls, setting_dict ):
        return cls( setting_dict )

    @classmethod
    def create_unlocked( cls, setting_dict ):
        s = cls( setting_dict )
        s._is_locked = False
        return s

    def __init__( self, setting_dict={} ):
        self._is_locked = False
        self._settings = {}
        #super( Settings, self ).__setattr__( "_is_locked", False )
        #super( Settings, self ).__setattr__( "_settings", {} )

        # Check if all setting names are valid all caps identifiers:
        for name in setting_dict:
            if not self.is_valid_setting_name( name ):
                raise ConfModuleError(
                    ConfModuleError.codes.invalid_setting_name,
                    safe_fmt( "'%s' is not a valid, all caps, non-internal identifier.", name ) )

           
        for name, value in setting_dict.items():
            self._settings[name] = value

        self._is_locked = True


    @classmethod
    def is_valid_setting_name( cls, name ):
        return is_all_caps_identifier_string( name ) and not is_internal_identifier_string( name )


    def lock( self ):
        self._is_locked = True


    def is_locked( self ):
        return self._is_locked


    def __contains__( self, item ):
        return item in self._settings


    def __getattr__( self, name ):
        if name in self._settings:
            return self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknown_setting_name,
                                   safe_fmt( "Unknown setting name: '%s'", name ) )


    def __setattr__( self, name, value ):
        if is_all_caps_identifier_string( name ) and not is_internal_identifier_string( name ):
            self.set_setting( name, value )
        else:
            super( Settings, self ).__setattr__( name, value )
        


    def __delattr__( self, name ):
        self.delete_setting( name )


    def __getitem__( self, name ):
        if name in self._settings:
            return self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknown_setting_name,
                                   safe_fmt( "Unknown setting name: '%s'", name ) )


    def __setitem__( self, name, value ):
        self.set_setting( name, value )


    def __delitem__( self, name ):
        self.delete_setting( name )


    def __len__( self ):
        return len(self._settings)


    def get_setting( self, name, default=None ):
        if name in self._settings:
            return self._settings[name]
        else:
            return default


    def set_setting( self, name, value ):
        if self._is_locked:
            raise ConfModuleError( ConfModuleError.codes.settings_locked, "Settings are locked." )
        elif not is_all_caps_identifier_string( name ) or is_internal_identifier_string( name ):
            raise ConfModuleError(
                ConfModuleError.codes.invalid_setting_name,
                safe_fmt( "'%s' is not a valid, all caps, non-internal identifier.", name ) )
        else:
            self._settings[name] = value


    def delete_setting( self, name ):
        if self._is_locked:
            raise ConfModuleError( ConfModuleError.codes.settings_locked, "Settings are locked." )
        elif name in self._settings:
            del self._settings[name]
        else:
            raise ConfModuleError( ConfModuleError.codes.unknown_setting_name,
                                   safe_fmt( "Unknown setting name: '%s'", name ) )


    def get_setting_count( self ):
        return len(self._settings)


    def update( self, updating_dict ):
        if self._is_locked:
            raise ConfModuleError( ConfModuleError.codes.settings_locked, "Settings are locked." )
        else:
            for name, value in updating_dict.items():
                self.set_setting( name, value )
