#
#  FILE NAME:
#      row.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import numbers

# P3 imports:
from p3.base.string_helpers import safe_fmt
from p3.data.cell import Cell
from p3.data.error import DataModuleError

_LOG = logging.getLogger( "p3.data" )


class Row(object):
    def __init__( self, table ):
        self._table = table
        self._cells = [Cell(col, self) for col in table.cols]


    @classmethod
    def is_cell_like( cls, cell ):
        return True

    # ===[ Operations ]========================================================================== #

    def add_column( self, col, value ):
        assert col.name is None or self._table.find_column( col.name ) is col

        cell = Cell( col, self, value )
        self._cells.append( cell )


    def fill_default( self ):
        for cell in self._cells:
            cell.value = cell.col.default


    def fill_by_position( self, cell_values ):
        for cell, value in zip(self._cells, cell_values):
            cell.value = value


    def fill_by_name( self, named_values ):
        for name, value in named_values:
            idx = self._table.get_column( name=name ).index
            self._cells[idx].value = value

    def remove_column( self, col_idx ):
        assert (col_idx >= 0) and (col_idx < len(self._cells))

        cell = self._cells.pop(col_idx)
        cell.row = None


    # ===[ Container ]=========================================================================== #

    def __contains__( self, key ):
        if isinstance( key, basestring ):
            return self.table.find_column( name=key ) is not None
        else:
            raise TypeError( safe_fmt( "The 'keys' argument is of type %s while only string is"
                                       " allowed.", type(key) ) )

    def __len__( self ):
        return len(self._cells)


    def __getitem__( self, key ):
        return self.get_cell( key ).value


    def __setitem__( self, key, value ):
        self.get_cell( key ).value = value


    def get_cell( self, key ):
        if isinstance( key, basestring ):
            cell_idx = self.table.get_column( name=key ).index

            assert cell_idx < len(self._cells)

            return self._cells[cell_idx]
        elif isinstance( key, numbers.Integral ):
            if (key >= 0) and (key < len(self._cells)):
                return self._cells[key]
            else:
                raise DataModuleError( DataModuleError.codes.invalid_column_index,
                                       safe_fmt( "Column index (%s) is out of valid range <0,%d)",
                                                 key, len(self._cells) ) )
        else:
            raise TypeError( safe_fmt( "The 'keys' argument is of type %s while only string or"
                                       " integer is allowed.", type(key) ) )

    # ===[ Properties & Conversions ]============================================================ #

    def __str__( self ):
        return ', '.join( [repr(c.value) for c in self._cells] )


    def cells():
        def fget( self ):
            return tuple(self._cells)
        def fset( self, cells ):
            assert self._cells is None, "cells property is already initialized"
            assert len(cells) == len(self._table.cols)
            assert not [c for c in cells if not self.is_cell_like(c)]
            self._cells = [c for c in cells]
        return locals()
    cells = property(**cells())


    def table():
        def fget( self ):
            return self._table
        return locals()
    table = property( **table() )
