#
#  FILE NAME:
#      execute.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging
import subprocess

# P3 imports:
from p3.base import safe_fmt
from p3.data import Table
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError


_LOG = logging.getLogger( "p3.db" )


## Execute specified query using MySQLdb interface.
def exec_query( db_conn, query, args=None ):
    try:
        db_conn.cursor().execute( query, args )
    except (db_conn.get_error_class(), db_conn.get_warning_class()) as e:
        _LOG.error( "Database API exception occured during execution of a query.\n"
                    "Query:\n%s\nArguments:\n%s\nException:\n%s", query, str(args), str(e) )
        raise
    _LOG.info( "Executed query:\n%s", db_conn.get_executed_query() )

    if db_conn.cursor().description is None:
        # This attribute [description] will be None for operations that
        # do not return rows [...]
        return None
    else:
        return Table.create_from_cursor( db_conn.cursor() )


## Execute specified query using spawned mysql process.
def exec_query_sh( db_conn, query ):
    if db_conn.dbms == DBMS.mysql:
        conn_args = (('--database=%s', db_conn.db_name),
                     ('--host=%s',     db_conn.db_host),
                     ('--password=%s', db_conn.db_pass),
                     ('--port=%s',     db_conn.db_port),
                     ('--user=%s',     db_conn.db_user))
        args = ['mysql']
        args.extend( [o % v for o, v in conn_args if v is not None] )

        sp = subprocess.Popen(
            args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE )
        out_data, err_data = sp.communicate( query )

        if sp.returncode:
            _LOG.error( "Script execution failed with error code %d and following messages:\n%s",
                        sp.returncode, out_data )
            raise DbModuleError( DbModuleError.codes.shell_execution_error, err_data )
    else:
        raise DbModuleError( DbModuleError.codes.dbms_not_supported,
                             safe_fmt( "Unsupported database management system: '%s'",
                                       '/'.join( DBMS.get_names( db_conn.dbms ) ) ) )
