#
#  FILE NAME:
#      create.py
#
#  AUTHOR(S):
#      Dariusz Walczak <dariusz.walczak@gmail.com>
#

# Standard library imports:
import logging

# P3 imports:
from p3.base import safe_fmt
from p3.db.dbms import DBMS
from p3.db.error import DbModuleError
from p3.db.manage.check import db_exists
from p3.db.manage.drop import drop_db


_LOG = logging.getLogger( "p3.db" )


## \param mng_conn DbConnection object which will be used for database management tasks;
#  \param db_name name of a database to be created;
#  \param drop_existing flag indicating that the function should drop existing database before
#      creation attempt;
def create_db( mng_conn, db_name, drop_existing=False ):
    _LOG.info( "Creating '%s' database...", db_name )

    assert isinstance( db_name, str )

    if drop_existing and db_exists( mng_conn, db_name ):
        drop_db( mng_conn, db_name )

    if mng_conn.dbms == DBMS.mysql:
        create_my_sql_db( mng_conn, db_name )
    elif mng_conn.dbms == DBMS.pgsql:
        create_pg_sql_db( mng_conn, db_name )
    else:
        msg = safe_fmt( "Unsupported database management system: '%s'",
                        '/'.join( DBMS.get_names( mng_conn.dbms ) ) )
        raise DbModuleError( DbModuleError.codes.dbms_not_supported, msg )


def create_my_sql_db( mng_conn, db_name ):
    query =  "CREATE DATABASE `{0:s}` CHARACTER SET = utf8;".format( db_name )
    mng_conn.cursor().execute( query )


def create_pg_sql_db( mng_conn, db_name ):
    import psycopg2.extensions

    # Python DB API specification expects that its implementation starts a transaction
    # automatically. On the other side, PostgreSQL's CREATE DATABASE statement mustn't be
    # executed inside of any transaction block (it causes 'InternalError: CREATE DATABASE
    # cannot run inside a transaction block' exception). To avoid it, following PostgreSQL's
    # specific extension must be used (set_isolation_level).

    prev_level = mng_conn.db_conn.isolation_level
    mng_conn.db_conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)
    query = "CREATE DATABASE \"{0:s}\";".format( db_name )
    mng_conn.cursor().execute( query )
    mng_conn.db_conn.set_isolation_level( prev_level )
